@@ -269,7 +269,9 @@ void config__init(struct mosquitto_db *db, struct mosquitto__config *config)
 void config__cleanup(struct mosquitto__config *config)
 {
 	int i;
+#ifdef WITH_BRIDGE
 	int j;
+#endif
 
 	mosquitto__free(config->clientid_prefixes);
 	mosquitto__free(config->persistence_location);
@@ -583,7 +585,9 @@ int config__read(struct mosquitto_db *db, struct mosquitto__config *config, bool
 	int rc = MOSQ_ERR_SUCCESS;
 	struct config_recurse cr;
 	int lineno = 0;
+#ifdef WITH_PERSISTENCE
 	int len;
+#endif
 	struct mosquitto__config config_reload;
 	int i;
 
@@ -766,6 +770,9 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						return MOSQ_ERR_INVAL;
 					}
 					while((token = strtok_r(NULL, " ", &saveptr))){
+						if (token[0] == '#'){
+							break;
+						}
 						cur_bridge->address_count++;
 						cur_bridge->addresses = mosquitto__realloc(cur_bridge->addresses, sizeof(struct bridge_address)*cur_bridge->address_count);
 						if(!cur_bridge->addresses){
@@ -1090,6 +1097,9 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 #else
 					log__printf(NULL, MOSQ_LOG_WARNING, "Warning: TLS support not available.");
 #endif
+				}else if(!strcmp(token, "check_retain_source")){
+					conf__set_cur_security_options(config, cur_listener, &cur_security_options);
+					if(conf__parse_bool(&token, "check_retain_source", &config->check_retain_source, saveptr)) return MOSQ_ERR_INVAL;
 				}else if(!strcmp(token, "ciphers")){
 #ifdef WITH_TLS
 					if(reload) continue; // Listeners not valid for reloading.
@@ -1327,7 +1337,10 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						cur_listener->security_options.allow_anonymous = -1;
 						cur_listener->protocol = mp_mqtt;
 						cur_listener->port = tmp_int;
-						token = strtok_r(NULL, "", &saveptr);
+						token = strtok_r(NULL, " ", &saveptr);
+						if (token != NULL && token[0] == '#'){
+							token = NULL;
+						}
 						mosquitto__free(cur_listener->host);
 						if(token){
 							cur_listener->host = mosquitto__strdup(token);
@@ -1871,6 +1884,9 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						}
 						token = strtok_r(NULL, " ", &saveptr);
 						if(token){
+							if (token[0] == '#'){
+								strtok_r(NULL, "", &saveptr);
+							}
 							cur_topic->qos = atoi(token);
 							if(cur_topic->qos < 0 || cur_topic->qos > 2){
 								log__printf(NULL, MOSQ_LOG_ERR, "Error: Invalid bridge QoS level '%s'.", token);
@@ -1880,8 +1896,11 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 							token = strtok_r(NULL, " ", &saveptr);
 							if(token){
 								cur_bridge->topic_remapping = true;
-								if(!strcmp(token, "\"\"")){
+								if(!strcmp(token, "\"\"") || token[0] == '#'){
 									cur_topic->local_prefix = NULL;
+									if (token[0] == '#'){
+										strtok_r(NULL, "", &saveptr);
+									}
 								}else{
 									if(mosquitto_pub_topic_check(token) != MOSQ_ERR_SUCCESS){
 										log__printf(NULL, MOSQ_LOG_ERR, "Error: Invalid bridge topic local prefix '%s'.", token);
@@ -1896,7 +1915,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 
 								token = strtok_r(NULL, " ", &saveptr);
 								if(token){
-									if(!strcmp(token, "\"\"")){
+									if(!strcmp(token, "\"\"") || token[0] == '#'){
 										cur_topic->remote_prefix = NULL;
 									}else{
 										if(mosquitto_pub_topic_check(token) != MOSQ_ERR_SUCCESS){
@@ -2011,21 +2030,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						log__printf(NULL, MOSQ_LOG_ERR, "Error: Invalid bridge configuration.");
 						return MOSQ_ERR_INVAL;
 					}
-					token = strtok_r(NULL, " ", &saveptr);
-					if(token){
-						if(cur_bridge->remote_username){
-							log__printf(NULL, MOSQ_LOG_ERR, "Error: Duplicate username value in bridge configuration.");
-							return MOSQ_ERR_INVAL;
-						}
-						cur_bridge->remote_username = mosquitto__strdup(token);
-						if(!cur_bridge->remote_username){
-							log__printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
-							return MOSQ_ERR_NOMEM;
-						}
-					}else{
-						log__printf(NULL, MOSQ_LOG_ERR, "Error: Empty username value in configuration.");
-						return MOSQ_ERR_INVAL;
-					}
+					if(conf__parse_string(&token, "bridge remote_username", &cur_bridge->remote_username, saveptr)) return MOSQ_ERR_INVAL;
 #else
 					log__printf(NULL, MOSQ_LOG_WARNING, "Warning: Bridge support not available.");
 #endif
